// ========================================================================
// vector4.h
//
// R^(4x1)
// ========================================================================

#ifndef	__VECTOR4_H
#define	__VECTOR4_H


#include	<math.h>
#include "vector3.h"

// | e1 |
// | e2 |
// | e3 |
// | e4 |

#define	_VECTOR_ROWMAX	4



#define	INLINE inline


//
//
struct vector4
{
friend struct vector3;
friend class matrix3x3;
friend class matrix4x4;
private:
	union{
		VAR   e[4];
		struct{
			VAR   x,y,z,w;
		};
	};


public:
	vector4() {}
	vector4(VAR   e0,VAR   e1,VAR   e2,VAR   e3){
		e[0]=e0;e[1]=e1;e[2]=e2;e[3]=e3;
	}
	vector4(const vector3& v) { *this=vector4(v.e[0],v.e[1],v.e[2],0); }
	vector4(const vector3& v,const VAR   f) { *this=vector4(v.e[0],v.e[1],v.e[2],f); }
	vector4(const vector4& v) { *this=vector4(v.e[0],v.e[1],v.e[2],v.e[3]); }
	~vector4() {}


	// one term operator
	VAR  & operator[](int n) { return e[n]; }

	// two term operator
	vector4& operator+=(vector4& a);
	vector4& operator-=(vector4& a);
	vector4& operator*=(VAR      a);
	vector4& operator/=(VAR      a);

	friend vector4 operator+(vector4& a,vector4& b);
	friend vector4 operator-(vector4& a,vector4& b);
	friend VAR     operator*(vector4& a,vector4& b);	//scalar product
	friend vector4 operator*(vector4& a,VAR      b);
	friend vector4 operator*(VAR      a,vector4& b);
	friend vector4 operator/(vector4& a,VAR      b);

	friend bool operator==(vector4& a,vector4& b);
	friend bool operator==(vector4& a,VAR      b);
	friend bool operator==(VAR      a,vector4& b);
	friend bool operator!=(vector4& a,vector4& b);
	friend bool operator!=(vector4& a,VAR      b);
	friend bool operator!=(VAR      a,vector4& b);


	// 
	VAR      dot   (vector4& a);
	vector4  cross3(vector4& a);
	VAR      norm();
	VAR      distance();
	vector4  unit();

	//
	void normalize();

};




inline vector4& vector4::operator+=(vector4& a)
{
	e[0]+=a.e[0];
	e[1]+=a.e[1];
	e[2]+=a.e[2];
	e[3]+=a.e[3];
	return *this;
}

inline vector4& vector4::operator-=(vector4& a)
{
	e[0]-=a.e[0];
	e[1]-=a.e[1];
	e[2]-=a.e[2];
	e[3]-=a.e[3];
	return *this;
}

inline vector4& vector4::operator*=(VAR      a)
{
	return *this=*this*a;
}

inline vector4& vector4::operator/=(VAR      a)
{
	return *this=*this/a;
}

// ========================================================================

inline vector4 operator+(vector4& a,vector4& b)
{
	return vector4( a.e[0]+b.e[0], a.e[1]+b.e[1], a.e[2]+b.e[2], a.e[3]+b.e[3] );
}


inline vector4 operator-(vector4& a,vector4& b)
{
	return vector4( a.e[0]-b.e[0], a.e[1]-b.e[1], a.e[2]-b.e[2], a.e[3]-b.e[3] );
}

inline VAR   operator*(vector4& a,vector4& b)
{
	return a.e[0]*b.e[0] + a.e[1]*b.e[1] + a.e[2]*b.e[2] + a.e[3]*b.e[3];
}

inline vector4 operator*(vector4& a,VAR      b)
{
	return vector4( a.e[0]*b, a.e[1]*b, a.e[2]*b, a.e[3]*b );
}

inline vector4 operator*(VAR      a,vector4& b)
{
	return b*a;
}

inline vector4 operator/(vector4& a,VAR      b)
{
	return a*(1.0f/b);
}

// ========================================================================

inline bool operator==(vector4& a,vector4& b)
{
	return a.e[0]==b.e[0] && a.e[1]==b.e[1] && a.e[2]==b.e[2] && a.e[3]==b.e[3];
}

inline bool operator==(vector4& a,VAR      b)
{
	return a.e[0]==b && a.e[1]==b && a.e[2]==b && a.e[3]==b;
}

inline bool operator==(VAR      a,vector4& b)
{
	return b==a;
}

inline bool operator!=(vector4& a,vector4& b)
{
	return !(a==b);
}

inline bool operator!=(vector4& a,VAR      b)
{
	return !(a==b);
}

inline bool operator!=(VAR      a,vector4& b)
{
	return !(a==b);
}

// ========================================================================

inline VAR   vector4::dot  (vector4& a)
{
	return a*a;
}

inline vector4 vector4::cross3(vector4& a)
{
	return  vector4(
			e[1]*a.e[2] -e[2]*a.e[1],
	        e[2]*a.e[0] -e[0]*a.e[2],
			e[0]*a.e[1] -e[1]*a.e[0],
			1
			);
}			

inline VAR   vector4::norm()
{
	return *this * *this;
}

inline VAR   vector4::distance()
{
	return sqrt(norm());
}

inline vector4 vector4::unit()
{
	return *this/distance();
}

// ========================================================================

inline void vector4::normalize()
{
	*this=unit();
}



#endif //__VECTOR4_H
